package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka monikulmioille
 * @author Markus Kivioja
 *
 */
public class Polygon extends BasicShape {

    protected RPoint[] ps;
    protected int count;


    public Polygon() {
        super();
    }


    /**
     * Luo monikulmion annetuista kulmapisteiden
     * x- ja y-koordinaateista
     * @param xpoints taulukko kulmapisteiden x-koordinaateista
     * @param ypoints taulukko kulmapisteiden y-koordinaateista
     */
    public Polygon(double[] xpoints, double[] ypoints) {
        super();
        if (xpoints.length < ypoints.length)
            this.count = xpoints.length;
        else
            this.count = ypoints.length;
        initialize();
        for (int i = 0; i < getCount(); i++)
            this.ps[i] = new RPoint(xpoints[i], ypoints[i]); // NOPMD, pakko
                                                             // luoda
    }


    /**
     * Luo monikulmion jonka kulmapisteiden x-koordinaatit lhtevt
     * annetusta pisteest x ja kasvavat siit annetun mrn dx,
     * y-koordinaatit annetaan taulukkona ypoints, esimerkiksi:
     * <pre>
     *     double[] ypoints = {0, 1, 2, 1, 0};
     *     Polygon p = new Polygon(0, 1, ypoints);
     *     
     *     Luo monikulmion jonka krkipisteet ovat (0,0), (1,1), (2,2), (3,1), (4,0) 
     * </pre>
     * @param x ensimmisen pisteen x-koordinaatti
     * @param dx pisteiden etisyys toisistaan x-akselin suunnassa
     * @param ypoints taulukko pisteiden y-koordinaateista
     */
    public Polygon(double x, double dx, double[] ypoints) {
        super();
        this.count = ypoints.length;
        initialize();
        for (int i = 0; i < getCount(); i++)
            this.ps[i] = new RPoint(x + i * dx, ypoints[i]); // NOPMD, pakko
                                                             // luoda
    }


    /**
     * Luo monikulmion annetuista kulmapisteiden
     * x-, y- ja z-koordinaateista
     * @param xpoints taulukko kulmapisteiden x-koordinaateista
     * @param ypoints taulukko kulmapisteiden y-koordinaateista
     * @param zpoints taulikko kulmapisteiden z-koordinaateista
     */
    public Polygon(double[] xpoints, double[] ypoints, double[] zpoints) {
        super();
        if (xpoints.length < ypoints.length)
            this.count = xpoints.length;
        else
            this.count = ypoints.length;
        initialize();
        for (int i = 0; i < this.count; i++)
            this.ps[i] = new RPoint(xpoints[i], ypoints[i], zpoints[i]); // NOPMD,
                                                                         // pakko
                                                                         // luoda
    }


    /**
     * Luo kaksi- tai kolmiulotteisen monikulmion annetuista kulmapisteist, esimerkiksi:
     * <pre>
     *      double[][] points = {{2, 6}, {5, 5}, {-1, 2.5}}
     *      Polygon p = new Poylgon(points);
     *      
     *      Luo tytetyn monikulmion jonka krkipisteet ovat (2, 6), (5, 5) ja (-1, 2.5)
     * </pre>
     * @param points 
     */
    public Polygon(double[][] points) {
        super();
        int dim = 4;
        for (int i = 0; i < points.length; i++) {
            if (points[i].length < dim)
                dim = points[i].length;
        }
        this.count = points.length;
        initialize();
        for (int i = 0; i < getCount(); i++) {
            if (dim == 2)
                this.ps[i] = new RPoint(points[i][0], points[i][1]); // NOPMD,
                                                                     // pakko
                                                                     // luoda
            if (dim == 3)
                this.ps[i] = new RPoint(points[i][0], points[i][1],
                        points[i][2]); // NOPMD, pakko luoda
        }
    }


    /**
     * Asettaa uudet kulmapisteet
     * @param points uudet kulmapisteet
     */
    public void setPoints(double[][] points) {
        int dim = 4;
        for (int i = 0; i < points.length; i++) {
            if (points[i].length < dim)
                dim = points[i].length;
        }
        this.count = points.length;
        initialize();
        for (int i = 0; i < this.count; i++) {
            if (dim == 2)
                this.ps[i] = new RPoint(points[i][0], points[i][1]); // NOPMD,
                                                                     // pakko
                                                                     // luoda
            if (dim == 3)
                this.ps[i] = new RPoint(points[i][0], points[i][1],
                        points[i][2]); // NOPMD, pakko luoda
        }
        redraw();
    }


    /**
     * Asettaa uudet kulmapisteet
     * @param points uudet kulmapisteet
     */
    public void setPoints(RPoint[] points) {
        this.count = points.length;
        initialize();
        for (int i = 0; i < this.count; i++) {
            this.ps[i] = points[i];
        }
        redraw();
    }


    /**
     * Luo monikulmion annetuista kulmapisteist jotka ovat RPoint olioita
     * @param points taulukko kulmapisteist jotka ovat RPoint-olioita
     */
    public Polygon(RPoint[] points) {
        super();
        this.count = points.length;
        initialize();
        for (int i = 0; i < points.length; i++) {
            this.ps[i] = points[i];
        }
    }


    /**
     * Luo monikulmion kulmapisteille oikean kokoisen RPoint-taulukon
     */
    protected void initialize() {
        this.ps = new RPoint[getCount()];
    }


    /**
     * Muuntaa monikulmion oikeat krkipisteet nytn pisteiksi annetulla muunnosmatriisilla
     * @param a muunnosmatriisi
     * @return monikulmion krkipisteiden koordinaatit nytll
     */
    protected int[][] makeScreenPoints(Matrix a) {
        int[][] sPoints = new int[2][getCount()];
        SPoint sp = new SPoint(0, 0);
        for (int i = 0; i < this.getCount(); i++) {
            a.transform(this.ps[i], sp);
            sPoints[0][i] = sp.getX();
            sPoints[1][i] = sp.getY();
        }
        return sPoints;
    }


    /**
     * Antaa krkipisteiden lukumrn
     * @return krkipisteiden lukumr
     */
    public final int getCount() {
        return count;
    }


    /**
     * Lis monikulmioon yhden krkipisteen joka on RPoint-oli
     * @param point listtv krkipiste
     * @return tm monikulmio lisyksen jlkeen
     */
    public Polygon add(RPoint point) {
        if (ps.length <= this.count) {
            RPoint newps[] = new RPoint[getCount() + 20];
            for (int i = 0; i < this.count; i++)
                newps[i] = ps[i];
            ps = newps;
        }
        ps[this.count++] = point;
        redraw();
        return this;
    }


    /**
     * Lis monikulmioon yhden krkipisteen
     * @param x listtvn krkipisteen x-koordinaatti
     * @param y listtvn krkipisteen y-koordinaatti
     * @return tm monikulmio lisyksen jlkeen
     */
    public Polygon add(double x, double y) {
        return add(new RPoint(x, y));
    }


    /**
     * Lis monikulmioon yhden krkipisteen
     * @param x listtvn krkipisteen x-koordinaatti
     * @param y listtvn krkipisteen y-koordinaatti
     * @param z listtvn krkipisteen z-koordinaatti
     * @return tm monikulmio lisyksen jlkeen
     */
    public Polygon add(double x, double y, double z) {
        return add(new RPoint(x, y, z));
    }


    @Override
    protected void drawShape(Graphics g, Matrix a) {
        int[][] sPoints = makeScreenPoints(a);
        g.drawPolygon(sPoints[0], sPoints[1], getCount());
    }
}
