package fi.jyu.mit.graphics;

/**
 * N:n asteen polynomi (N <= 15)
 * @author Vesa Lappalainen
 *
 */
public class Polynom implements FunctionRR {

    private double k[] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    private int deg = -1;


    /**
     * Luo uuden ensimmisen asteen polynomifunktion
     * @param a ensimmisen asteen termin kerroin
     * @param b vakiotermin kerroin
     */
    public Polynom(double a, double b) {
        deg = 1;
        k[0] = b;
        k[1] = a;
    }


    /**
     * Luo uuden toisen asteen polynomifunktion
     * @param a toisen asteen termin kerroin
     * @param b ensimmisen asteen termin kerroin
     * @param c vakiotermin kerroin
     */
    public Polynom(double a, double b, double c) {
        deg = 2;
        k[0] = c;
        k[1] = b;
        k[2] = a;
    }


    /**
     * Luo uuden kolmannen asteen polynomifunktion
     * @param a kolmannen asteen termin kerroin
     * @param b toisen asteen termin kerroin
     * @param c ensimmisen asteen termin kerroin
     * @param d vakiotermin kerroin
     */
    public Polynom(double a, double b, double c, double d) {
        deg = 3;
        k[0] = d;
        k[1] = c;
        k[2] = b;
        k[3] = a;
    }


    /**
     * Luo uuden neljnnen asteen polynomifunktion
     * @param a neljnnen asteen termin kerroin
     * @param b kolmannen asteen termin kerroin
     * @param c toisen asteen termin kerroin
     * @param d ensimmisen asteen termin kerroin
     * @param e vakiotermin kerroin
     */
    public Polynom(double a, double b, double c, double d, double e) {
        deg = 4;
        k[0] = e;
        k[1] = d;
        k[2] = c;
        k[3] = b;
        k[4] = a;
    }


    /**
     * Asettaa n:n asteen termin kertoimen (n<=5)
     * @param n aste (<=15)
     * @param a n:n asteen termin kerroin
     */
    public void add(int n, double a) {
        if (n >= k.length)
            return;
        if (n > deg)
            deg = n;
        k[n] = a;
    }


    /**
     * Asettaa n:n asteen termin kertoimen 
     * @param n aste (<=15)
     * @param a n:n asteen termin kerroin
     */
    public void set(int n, double a) {
        add(n, a);
    }


    /**
     * Palauttaa n:n asteen termin kertoimen 
     * @param n aste (<=15)
     * @return n:n termin kerroin
     */
    public double get(int n) {
        if (n > deg)
            return 0;
        return k[n];
    }


    /**
     * Palauttaa polynoimin asteluvun 
     * @return n:n termin kerroin
     */
    public int getDeg() {
        return deg;
    }


    /**
     * Polynomi funktion arvo pisteess x 
     * Lasketaan polynomi ax2 + bx +c
     * muodossa (ax+b)*x + c
     * @param x mist polynomin arvo lasketaan
     * @return polynomin arvo pisteess x
     * 
     */
    @Override
    public double f(double x) { // NOPMD, nyt f on hyv nimi
        if (deg == -1)
            return 0;
        double s = k[deg];
        for (int i = deg - 1; i >= 0; i--)
            s = s * x + k[i];
        return s;
    }
}
