package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka tekstille
 * @author Markus Kivioja
 *
 */
public class Text extends BasicShape {

    String text;
    RPoint leftBottom;
    double tx = 1.0;
    double ty = 1.0;


    /**
     * Luo uuden tekstin jonka z-koordinaatti on 0
     * @param text piirrettv teksti
     * @param x vasemman alakulman x-koordinaatti
     * @param y vasemman alakulman y-koordinaatti
     */
    public Text(String text, double x, double y) {
        super();
        this.text = text;
        this.leftBottom = new RPoint(x, y);
    }


    /**
     * Asetetaan tekstin suhteellinen siirto
     * @param sx -1 = oikea reuna, 0 = keskitys, 1 = vasen reuna
     * @param sy -1 = ylnurkka,   0 = keskitys, 1 = alanurkka
     */
    public void setTranslate(double sx, double sy) {
        tx = sx;
        ty = sy;
    }


    /**
     * Luo uuden tekstin
     * @param text piirrettv teksti
     * @param x vasemman alakulman x-koordinaatti
     * @param y vasemman alakulman y-koordinaatti
     * @param z vasemman alakulman z-koordinaatti
     */
    public Text(String text, double x, double y, double z) {
        super();
        this.text = text;
        this.leftBottom = new RPoint(x, y, z);
    }


    /**
     * Piirretn teksti
     * @see fi.jyu.mit.graphics.BasicShape#drawShape(java.awt.Graphics, fi.jyu.mit.graphics.Matrix)
     */
    @Override
    protected void drawShape(Graphics g, Matrix a) {
        FontMetrics fm = g.getFontMetrics();
        int w = fm.stringWidth(text);
        int h = fm.getAscent();
        SPoint sp = a.transform(this.leftBottom);
        g.drawString(this.text, (int) (sp.getX() + w / 2.0 * (tx - 1)),
                (int) (sp.getY() - h / 2.0 * (ty - 1)));
    }
}
