package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka ellipseille
 * @author Markus Kivioja
 *
 */
public class Ellipse extends BasicShape {

    private final RPoint center;
    private double a, b;
    private int resolution;
    private int[] sXPoints;
    private int[] sYPoints;


    /**
     * Luo uuden ellipsin
     * @param x ellipsin keskipisteen x-koordinaatti
     * @param y ellipsin keskipisteen y-koordinaatti
     * @param a x-suuntaisen puoliakselin pituus
     * @param b y-suuntaisen puoliakselin pituus
     */
    public Ellipse(double x, double y, double a, double b) {
        super();
        this.center = new RPoint(x, y);
        this.a = a;
        this.b = b;
        initialize();
    }


    /**
     * Luo uuden ellipsin
     * @param x ellipsin keskipisteen x-koordinaatti
     * @param y ellipsin keskipisteen y-koordinaatti
     * @param z ellipsin keskipisteen z-koordinaatti
     * @param a x-suuntaisen puoliakselin pituus
     * @param b y-suuntaisen puoliakselin pituus
     */
    public Ellipse(double x, double y, double z, double a, double b) {
        super();
        this.center = new RPoint(x, y, z);
        this.a = a;
        this.b = b;
        initialize();
    }


    /**
     * Alustaa ellipsin asetukset
     */
    private void initialize() {
        this.resolution = 360;
        this.sXPoints = new int[resolution];
        this.sYPoints = new int[resolution];
    }


    /**
     * Piirret��n ellipsi
     * @see fi.jyu.mit.graphics.BasicShape#drawShape(java.awt.Graphics, fi.jyu.mit.graphics.Matrix)
     */
    @SuppressWarnings("hiding")
    @Override
    protected void drawShape(Graphics g, Matrix a) {
        Vector rp = new Vector(); 
        SPoint sp = new SPoint(0, 0);
        for (int i = 0; i < resolution; i++) {
            a.transform(
                    this.a * Math.cos((2 * Math.PI / resolution) * i)
                            + this.center.getX(),
                    this.b * Math.sin((2 * Math.PI / resolution) * i)
                            + this.center.getY(),
                    this.center.getZ(), rp, sp);
            sXPoints[i] = sp.getX();
            sYPoints[i] = sp.getY();
        }
        g.drawPolygon(sXPoints, sYPoints, resolution);
    }
}
