package fi.jyu.mit.graphics;

import java.awt.*;

/**
 * Luokka suorakaiteelle
 * @author vesal
 *
 */
public class Rect extends BasicShape {

    private final RPoint center;
    private double a, b;
    private int[] sXPoints = new int[4];
    private int[] sYPoints = new int[4];
    private RPoint[] rpoints = new RPoint[4];


    /**
     * Luo uuden suorakaiteen
     * @param x suorakaiteen keskipisteen x-koordinaatti
     * @param y suorakaiteen keskipisteen y-koordinaatti
     * @param a x-suuntaisen puoliakselin pituus
     * @param b y-suuntaisen puoliakselin pituus
     */
    public Rect(double x, double y, double a, double b) {
        super();
        this.center = new RPoint(x, y);
        this.a = a;
        this.b = b;
        initialize();
    }


    /**
     * Luo uuden suorakaiteen
     * @param x suorakaiteen keskipisteen x-koordinaatti
     * @param y suorakaiteen keskipisteen y-koordinaatti
     * @param z suorakaiteen keskipisteen z-koordinaatti
     * @param a x-suuntaisen puoliakselin pituus
     * @param b y-suuntaisen puoliakselin pituus
     */
    public Rect(double x, double y, double z, double a, double b) {
        super();
        this.center = new RPoint(x, y, z);
        this.a = a;
        this.b = b;
        initialize();
    }


    /**
     * Alustaa suorakaiteen asetukset
     */
    private void initialize() {
        rpoints[0] = center.sum(a, -b);
        rpoints[1] = center.sum(a, b);
        rpoints[2] = center.sum(-a, b);
        rpoints[3] = center.sum(-a, -b);
    }


    /**
     * Piirret��n suorakaide
     * @see fi.jyu.mit.graphics.BasicShape#drawShape(java.awt.Graphics, fi.jyu.mit.graphics.Matrix)
     */
    @SuppressWarnings("hiding")
    @Override
    protected void drawShape(Graphics g, Matrix a) {
        Vector rp = new Vector();
        SPoint sp = new SPoint(0, 0);
        for (int i = 0; i < 4; i++) {
            a.transform(rpoints[i].getX(), rpoints[i].getY(), rpoints[i].getZ(),
                    rp, sp);
            sXPoints[i] = sp.getX();
            sYPoints[i] = sp.getY();
        }
        g.drawPolygon(sXPoints, sYPoints, 4);
    }
}
