package fi.jyu.mit.graphics;

/**
 * Neliskulmio jonka krkipisteet kaksiulotteisia kokonaislukupisteit
 * @author Markus Kivioja
 *
 */
public class SRect {

    private final SPoint p1, p2;


    /**
     * Luo uuden neliskulmion jonka kulmapisteet ovat SPoint-olioita
     * @param leftTop vasen ylkulma
     * @param rightBottom oikea alakulma
     */
    public SRect(SPoint leftTop, SPoint rightBottom) {
        this.p1 = leftTop;
        this.p2 = rightBottom;
    }


    /**
     * Luo uuden neliskulmion kytten annettuja kulmien koordinaatteja
     * @param left vasemman reunan x-koordinaatti
     * @param top ylreunan y-koordinaatti
     * @param right oikean reunan x-koordinaatti
     * @param bottom alareunan y-koordinaatti
     */
    public SRect(int left, int top, int right, int bottom) {
        this.p1 = new SPoint(left, top);
        this.p2 = new SPoint(right, bottom);
    }


    /**
     * Antaa tmn nelikulmion vasemman ylkulman SPoint-oliona
     * @return nelikulmion vasen ylkulma
     */
    public SPoint getP1() {
        return p1;
    }


    /**
     * Antaa tmn nelikulmion oikean alakulman SPoint-oliona
     * @return nelikulmion oikea alakulma
     */
    public SPoint getP2() {
        return p2;
    }


    /**
     * Antaa tmn nelikulmion leveyden
     * @return nelikulmion leveys
     */
    public int width() {
        return p2.getX() - p1.getX();
    }


    /**
     * Antaa tmn nelikulmion korkeuden
     * @return nelikulmion korkeus
     */
    public int height() {
        return p2.getY() - p1.getY();
    }


    /**
     * Antaa tmn nelikulmion keskipisteen x-koordinaatin
     * @return nelikulmion keskipisteen x-koordinaatti
     */
    public double midX() {
        return (p1.getX() + p2.getX()) / 2.0;
    }


    /**
     * Antaa tmn nelikulmion keskipisteen y-koordinaatin
     * @return nelikulmion keskipisteen y-koordinaatti
     */
    public double midY() {
        return (p1.getY() + p2.getY()) / 2.0;
    }
}
