package fi.jyu.mit.graphics;

/**
 * Luokka 4x1 vektoreille
 * @author Markus Kivioja
 *
 */
public class Vector {

    private final static int VSIZE = 4;
    private double[] ar = new double[VSIZE];


    /**
     * Luo uuden 4x1 vektorin (0.0, 0.0, 0.0, 1.0)T
     *
     */
    public Vector() {
        ar[0] = 0.0;
        ar[1] = 0.0;
        ar[2] = 0.0;
        ar[3] = 1.0;
    }


    /**
     * Luo uuden 4x1 vektorin annetuista alkioista siten, ett viimeinen alkio on 1.0
     * @param a0 alkio (0, 0)
     * @param a1 alkio (1, 0)
     * @param a2 alkio (2, 0)
     */
    public Vector(double a0, double a1, double a2) {
        ar[0] = a0;
        ar[1] = a1;
        ar[2] = a2;
        ar[3] = 1.0;
    }


    /**
     * Luo uuden 4x1 vektorin jossa annetussa paikassa on annettu alkio
     * @param i paikka johon alkio asetetaan
     * @param ai asetettava alkio
     */
    public Vector(int i, double ai) {
        for (int j = 0; j < VSIZE; j++)
            ar[j] = 0.0;
        ar[i] = ai;
    }


    /**
     * Asettaa vektorin annettuun paikkaan annetun alkion
     * @param i paikka johon alkio asetetaan
     * @param ai asetettava alkio
     * @return tm vektori muutoksen jlkeen
     */
    public Vector set(int i, double ai) { 
        this.ar[i] = ai;
        return this;
    }


    /**
     * Asettaa vektorin kolme ensimmist alkiota
     * @param x alkio (0, 0)
     * @param y alkio (1, 0)
     * @param z alkio (2, 0)
     * @return tm vektori muutoksen jlkeen
     */
    public Vector set(double x, double y, double z) { 
        this.ar[0] = x;
        this.ar[1] = y;
        this.ar[2] = z;
        return this;
    }


    /**
     * Asettaa vektorin kaksi ensimmist alkiota
     * @param x alkio (0, 0)
     * @param y alkio (1, 0)
     * @return tm vektori muutoksen jlkeen
     */
    public Vector set(double x, double y) { 
        this.ar[0] = x;
        this.ar[1] = y;
        return this;
    }


    /**
     * Laskee tmn ja annetun vektorin sistulon
     * @param b vektori jonka kanssa sistulo lasketaan
     * @return tmn ja annetun vektorin sistulo
     */
    public double innerProduct(Vector b) { 
        double s = 0;
        for (int i = 0; i < VSIZE; i++)
            s += ar[i] * b.ar[i];
        return s;
    }


    /**
     * Kertoo tt vektoria annetulla skalaarilla
     * @param y kertova skalaari
     * @return tm vektori kerrottuna annetulla skalaarilla
     */
    public Vector scalarProduct(double y) { 
        Vector p = new Vector(0, 0, 0); 
        for (int i = 0; i < VSIZE; i++)
            p.ar[i] = this.ar[i] * y;
        return p;
    }


    /**
     * Summaa tähän vektoriin annetun vektorin ja palauttaa tuloksen
     * @param v summattava vektori
     * @return tämän vektorin ja annetun vektorin summa
     */
    public Vector sum(Vector v) {
        Vector sum = new Vector();
        for (int i = 0; i < VSIZE; i++) {
            sum.set(i, this.a(i) + v.a(i));
        }
        return sum;
    }


    /**
     * Palauttaa tmn vektorin annetussa paikassa olevan alkion
     * @param i paikka josta alkio halutaan
     * @return annetussa paikassa oleva alkio
     */
    public double a(int i) {
        return this.ar[i];
    }
}
